"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const decorators_1 = require("../common/decorators");
const os_1 = require("os");
const run_controller_1 = require("./run-controller");
class DebugController extends run_controller_1.RunController {
    constructor($analyticsService, $buildDataService, $buildController, $debugDataService, $debugService, $deviceInstallAppService, $devicesService, $errors, $hmrStatusService, $hooksService, $liveSyncServiceResolver, $logger, $platformsDataService, $pluginsService, $prepareController, $prepareDataService, $prepareNativePlatformService, $projectDataService, $runEmitter) {
        super($analyticsService, $buildDataService, $buildController, $deviceInstallAppService, $devicesService, $errors, $hmrStatusService, $hooksService, $liveSyncServiceResolver, $logger, $platformsDataService, $pluginsService, $prepareController, $prepareDataService, $prepareNativePlatformService, $projectDataService, $runEmitter);
        this.$debugDataService = $debugDataService;
        this.$debugService = $debugService;
    }
    enableDebugging(projectData, deviceDescriptor, refreshInfo) {
        return __awaiter(this, void 0, void 0, function* () {
            const { debugOptions } = deviceDescriptor;
            debugOptions.start = !debugOptions.debugBrk;
            debugOptions.forceDebuggerAttachedEvent = refreshInfo.didRestart;
            const deviceOption = {
                deviceIdentifier: deviceDescriptor.identifier,
                debugOptions: debugOptions,
            };
            return this.enableDebuggingCoreWithoutWaitingCurrentAction(deviceOption, deviceDescriptor, { projectDir: projectData.projectDir });
        });
    }
    attachDebugger(settings) {
        return __awaiter(this, void 0, void 0, function* () {
            if (settings.debugOptions) {
                settings.debugOptions.chrome = settings.debugOptions.chrome === undefined ? true : settings.debugOptions.chrome;
                settings.debugOptions.start = settings.debugOptions.start === undefined ? true : settings.debugOptions.start;
            }
            else {
                settings.debugOptions = {
                    chrome: true,
                    start: true
                };
            }
            const projectData = this.$projectDataService.getProjectData(settings.projectDir);
            const debugData = this.$debugDataService.createDebugData(projectData, { device: settings.deviceIdentifier });
            const debugInfo = yield this.$debugService.debug(debugData, settings.debugOptions);
            const result = this.printDebugInformation(debugInfo, settings.debugOptions.forceDebuggerAttachedEvent);
            return result;
        });
    }
    printDebugInformation(debugInformation, fireDebuggerAttachedEvent = true) {
        if (!!debugInformation.url) {
            if (fireDebuggerAttachedEvent) {
                this.$runEmitter.emitDebuggerAttachedEvent(debugInformation);
            }
            this.$logger.info(`To start debugging, open the following URL in Chrome:${os_1.EOL}${debugInformation.url}${os_1.EOL}`.cyan);
        }
        return debugInformation;
    }
    refreshApplication(projectData, liveSyncResultInfo, deviceDescriptor) {
        const _super = Object.create(null, {
            refreshApplication: { get: () => super.refreshApplication }
        });
        return __awaiter(this, void 0, void 0, function* () {
            liveSyncResultInfo.waitForDebugger = deviceDescriptor.debugOptions && deviceDescriptor.debugOptions.debugBrk;
            const refreshInfo = yield _super.refreshApplication.call(this, projectData, liveSyncResultInfo, deviceDescriptor);
            yield this.enableDebugging(projectData, deviceDescriptor, refreshInfo);
            return refreshInfo;
        });
    }
    enableDebuggingCoreWithoutWaitingCurrentAction(deviceOption, deviceDescriptor, debuggingAdditionalOptions) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!deviceDescriptor) {
                this.$errors.failWithoutHelp(`Couldn't enable debugging for ${deviceOption.deviceIdentifier}`);
            }
            deviceDescriptor.debuggingEnabled = true;
            deviceDescriptor.debugOptions = deviceOption.debugOptions;
            const currentDeviceInstance = this.$devicesService.getDeviceByIdentifier(deviceOption.deviceIdentifier);
            const attachDebuggerOptions = {
                deviceIdentifier: deviceOption.deviceIdentifier,
                isEmulator: currentDeviceInstance.isEmulator,
                outputPath: deviceDescriptor.outputPath,
                platform: currentDeviceInstance.deviceInfo.platform,
                projectDir: debuggingAdditionalOptions.projectDir,
                debugOptions: deviceOption.debugOptions
            };
            let debugInformation;
            try {
                debugInformation = yield this.attachDebugger(attachDebuggerOptions);
            }
            catch (err) {
                this.$logger.trace("Couldn't attach debugger, will modify options and try again.", err);
                attachDebuggerOptions.debugOptions.start = false;
                try {
                    debugInformation = yield this.attachDebugger(attachDebuggerOptions);
                }
                catch (innerErr) {
                    this.$logger.trace("Couldn't attach debugger with modified options.", innerErr);
                    throw err;
                }
            }
            return debugInformation;
        });
    }
}
__decorate([
    decorators_1.performanceLog()
], DebugController.prototype, "enableDebugging", null);
__decorate([
    decorators_1.performanceLog()
], DebugController.prototype, "enableDebuggingCoreWithoutWaitingCurrentAction", null);
exports.DebugController = DebugController;
$injector.register("debugController", DebugController);
